﻿using RimWorld;
using Verse;

namespace VREAndroids
{
    public class Gene_RainVulnerability : Gene
    {
        public static readonly SimpleCurve rainVulnerabilityPerPowerChance = new SimpleCurve
        {
            new CurvePoint(0.5f, 0f),
            new CurvePoint(0.75f, 0.035f),
            new CurvePoint(1f, 0.05f)
        };
        public override void Tick()
        {
            base.Tick();
            if (pawn.Spawned && pawn.IsHashIntervalTick(600) && pawn.Map.weatherManager.RainRate >= 0.01f 
                && pawn.Position.Roofed(pawn.Map) is false)
            {
                var hediff = pawn.health.hediffSet.GetFirstHediffOfDef(VREA_DefOf.VREA_Reactor) as Hediff_AndroidReactor;
                if (hediff != null)
                {
                    if (Rand.Chance(rainVulnerabilityPerPowerChance.Evaluate(hediff.Energy)))
                    {
                        TaggedString taggedString = "VREA.AndroidShortCircuitRain".Translate(pawn.Named("PAWN"), pawn);
                        TargetInfo targetInfo = new TargetInfo(pawn.Position, pawn.Map);
                        if (pawn.Faction == Faction.OfPlayer)
                        {
                            Find.LetterStack.ReceiveLetter("LetterLabelShortCircuit".Translate(), taggedString, LetterDefOf.NegativeEvent, targetInfo);
                        }
                        else
                        {
                            Messages.Message(taggedString, targetInfo, MessageTypeDefOf.NeutralEvent);
                        }
                        GenExplosion.DoExplosion(pawn.OccupiedRect().RandomCell, pawn.Map, 1.9f, DamageDefOf.Flame, null);
                        hediff.Energy -= 0.05f;
                    }
                }
            }
        }
    }
}
